
from cement import App, TestApp, init_defaults
from cement.core.exc import CaughtSignal
from .core.exc import RemovalistError
from .controllers.base import Base
from .controllers.dependencies import Dependencies
from .controllers.template import Template

CONFIG = init_defaults('removalist')
CONFIG['removalist']['dependencies_path'] = '.dependencies'
CONFIG['removalist']['template_extension'] = 'rcl'


class Removalist(App):
    """Removalist for Hashicorp Packer primary application."""

    class Meta:
        label = 'removalist'
        config_defaults = CONFIG
        close_on_exit = True
        config_handler = 'yaml'
        config_file_suffix = '.yml'
        log_handler = 'colorlog'
        output_handler = 'jinja2'
        extensions = [
            'yaml',
            'colorlog',
            'jinja2',
        ]
        handlers = [
            Base,
            Dependencies,
            Template
        ]


class RemovalistTest(TestApp, Removalist):
    """A sub-class of Removalist that is better suited for testing."""

    class Meta:
        label = 'removalist'


def main():
    with Removalist() as app:
        try:
            app.run()

        except AssertionError as e:
            print('AssertionError > %s' % e.args[0])
            app.exit_code = 1

            if app.debug is True:
                import traceback
                traceback.print_exc()

        except RemovalistError as e:
            print('RemovalistError > %s' % e.args[0])
            app.exit_code = 1

            if app.debug is True:
                import traceback
                traceback.print_exc()

        except CaughtSignal as e:
            print('\n%s' % e)
            app.exit_code = 0


if __name__ == '__main__':
    main()



from sys import exit
from cement import Controller, ex
from .helpers.loader import json as loader_json
from .helpers.data import root as data_root
from .helpers.data import dependencies as data_dependencies
from .helpers.data import merge as data_merge
from .helpers.serializer import json as serializer_json


class Template(Controller):
    class Meta:
        label = 'template'
        stacked_type = 'nested'
        stacked_on = 'base'

    @ex(
        help='compile template',
        arguments=[
            ([],
             {'help': 'removalist template to compile',
              'action': 'store',
              'dest': 'template'}),
        ]
    )
    def compile(self):
        name = self.app.pargs.template
        data = {}

        while name is not None:
            try:
                self.app.log.info("merging {}.".format(name))
                template = loader_json(name)
                name = data_root(template)
                data = data_merge(data, template)
            except TypeError:
                self.app.log.error("cannot load {}.".format(name))
                exit(1)

        dependencies = data_dependencies(data)
        for dependency in dependencies:
            try:
                self.app.log.info("merging {}.".format(dependency))
                node = loader_json(dependency)
                data = data_merge(node, data)
            except TypeError:
                self.app.log.error("cannot load {}.".format(dependency))
                exit(1)

        serializer_json(data)



from sys import exit
from os import getcwd
from cement import Controller, ex
from .helpers.loader import yaml as loader_yaml
from .helpers.data import repository as data_repository
from .helpers.data import name as data_name
from .helpers.data import version as data_version
from .helpers.git import clone as git_clone
from .helpers.git import checkout as git_checkout


class Dependencies(Controller):
    class Meta:
        label = 'dependencies'
        stacked_type = 'nested'
        stacked_on = 'base'

    @ex(
        help='install dependencies',
        arguments=[
            (['-f', '--file'],
             {'help': 'custom requirements file',
              'action': 'store',
              'dest': 'file',
              'default': 'requirements'}),
        ]
    )
    def install(self):
        requirements = loader_yaml(self.app.pargs.file)

        if requirements is None:
            self.app.log.error("cannot load {}.".format(self.app.pargs.file))
            exit(1)

        for requirement in requirements:
            repository = data_repository(requirement)
            name = data_name(requirement)
            version = data_version(requirement)

            if repository is None:
                self.app.log.warning("missing repository key, skipping")
                continue

            if name is None:
                self.app.log.warning("missing name key, skipping")
                continue

            path = "{}/{}".format(getcwd(), self.app.config.get(
                'removalist',
                'dependencies_path')
            )
            self.app.log.info("installing {}@{}".format(
                name,
                version or "master")
            )
            git_clone(path, name, repository)

            if version is not None:
                git_checkout(path, name, version)



from cement import Controller
from cement.utils.version import get_version_banner
from ..core.version import get_version

VERSION_BANNER = """
Add Dependencies and Extensions for Hashicorp Packer %s
%s
""" % (get_version(), get_version_banner())


class Base(Controller):
    class Meta:
        label = 'base'
        description = 'Add Dependencies and Extensions for Hashicorp Packer'
        arguments = [
            (['-v', '--version'],
             {'action': 'version',
              'version': VERSION_BANNER}),
        ]

    def _default(self):
        """Default action if no sub-command is passed."""

        self.app.args.print_help()




from setuptools import setup, find_packages
from removalist.core.version import get_version

VERSION = get_version()

f = open('README.md', 'r')
LONG_DESCRIPTION = f.read()
f.close()

setup(
    name='removalist',
    version=VERSION,
    description='Add Dependencies and Extensions for Hashicorp Packer',
    long_description=LONG_DESCRIPTION,
    long_description_content_type='text/markdown',
    author='Bruno Briante',
    author_email='bruno.briante@convenia.com.br',
    url='https://github.com/convenia/removalist',
    license='MIT',
    packages=find_packages(exclude=['ez_setup', 'tests*']),
    package_data={'removalist': ['templates/*']},
    include_package_data=True,
    entry_points="""
        [console_scripts]
        removalist = removalist.main:main
    """,
)
